#original frequency dataset legend
#  
#This file contains the frequencies of side effects 
#as extracted from the labels. Format:
#  
#1 & 2: STITCH compound ids (flat/stereo, see above)
#3: UMLS concept id as it was found on the label
#4: "placebo" if the info comes from placebo administration, "" otherwise
#5: a description of the frequency: for example "postmarketing", "rare", 
#"infrequent", "frequent", "common", or an exact percentage
#6: a lower bound on the frequency
#7: an upper bound on the frequency
#8-10: MedDRA information as for meddra_all_se.tsv.gz
#
#The bounds are ranges like 0.01 to 1 for "frequent". If the exact frequency 
#is known, then the lower bound matches the upper bound. Due to the nature 
#of the data, there can be more than one frequency for the same label,
#e.g. from different clinical trials or for different levels of severeness.

library(data.table)
library(tidyverse)
library(igraph)
library(networkD3)
library(dplyr)
library(htmlwidgets)
library(RColorBrewer)

#-------------------------------------------------------------------------------
#preprocessing all data 
drug_names<-as.data.frame(fread("drug_names.tsv", header=FALSE))
side_effects<-as.data.frame(fread("meddra_freq.tsv", header=FALSE))

colnames(drug_names) <- c("code", "name")

side_effects$V2 <- NULL #removing undesirable columns
side_effects$V3 <- NULL
side_effects$V8 <- NULL
side_effects$V9 <- NULL
colnames(side_effects) <- c("code", 
                            "placebo", 
                            "freq_description", 
                            "freq_lower_bound", 
                            "freq_upper_bound", 
                            "effect_description")

side_effects_crop <- side_effects[1:100000, ]

join_df <- merge(drug_names, side_effects_crop, by = "code") #merging datasets
df <- join_df %>% distinct() %>% distinct(effect_description, .keep_all = TRUE)
   #removing all duplicate rows

#-------------------------------------------------------------------------------
#10 most frequent side_effects simple pie visualization
keep <- c(tail(names(sort(table(df$effect_description))), 10))
keep_df <- df[df$effect_description %in% keep, ] 

ggplot(keep_df,aes(x='',y=freq_upper_bound,fill=effect_description)) + 
  geom_bar(stat = "Identity",width = 2,color="Yellow") + 
  coord_polar("y",start = 0) +
  theme_void()

#-------------------------------------------------------------------------------
#interactive network graph based on picked substance
selected_substance <- "carnitine"

selected_df <- df[df$name==selected_substance,c("name", "effect_description")]
   #cropping original dataframe to fit the network graph desired input 
   #and only keeping side effects for the chosen substance

n <- simpleNetwork(selected_df, height="100px", width="100px", Source = 1, 
                   Target = 2, linkDistance = 10, charge = -900, fontSize = 14, 
                   linkColour = "#666", nodeColour = "#69b3a2", opacity = 0.9, 
                   zoom = T)
   #if we choose df as source data frame, source = 2, target = 7, we can see
   #a network including all of the substances and their side effects but
   #its very very slow and cant be explored properly
n
#saveWidget(n, file=paste0(getwd(), "/networkInteractiveSingleNetwork.html"))
   #we can save the interactive widget for later


#-------------------------------------------------------------------------------
#non interactive network graph with special attributes for the side effect nodes
# - selected substance is in the middle of all side effects
# - side effects can be of two different colors depending it they are marked as
#   placebo or not in the original dataset
# - size of the side effect nodes is determined by the upper bound of their 
#   frequency (since the frequencies are already quite small, very rarely 
#   surpassing 0.3 [on a scale of 0.01 to 1], I chose the upper bound as it is
#   the highest it can get)
# - there are only simple labels on the nodes for easy quick understanding and
#   simple visualization 
selected_df <- df[df$name==selected_substance,c("name",     
                                                "effect_description",
                                                "placebo", 
                                                "freq_upper_bound")]
    #again cutting the data frame but this time Im keeping the placebo and
    #upper bound of frequency attributes

selected_df$placebo <- ifelse(selected_df$placebo == "", "genuine", "placebo")

g <- graph_from_data_frame(selected_df, directed=F) 

par(mar=c(0,0,0,0))

#when I tried vertex mapping without moving the indexes in a for cycle, 
#the values were always moved by one row and the final graph was then incorrect
#and not corresponding to the actual data, so I decided to take care of it 
#by using a for cycle
for(index in 2:nrow(selected_df)+1){
  V(g)$color[index] <- ifelse(selected_df$placebo[index-1] == "genuine", 
                              "turquoise1", "maroon1")
  V(g)$size[index] <- 70*selected_df$freq_upper_bound[index-1]
}

V(g)$size[1] <- 10
plot(g,
     vertex.label.cex=0.8,
     vertex.frame.color="transparent"
)

#adding a simple legend for the colors
legend(x=1, y=1, 
       legend=c("genuine", "placebo"), 
       col = c("turquoise1", "maroon1"), 
       bty = "n", pch=20 , pt.cex = 2, cex = 1,
       horiz = F)
